package com.tzld.longarticle.recommend.server.service.rank;


import com.tzld.longarticle.recommend.server.service.score.ScoreParam;
import com.tzld.longarticle.recommend.server.service.score.ScoreResult;
import com.tzld.longarticle.recommend.server.service.score.ScoreService;
import com.tzld.longarticle.recommend.server.service.score.ScoreStrategy;
import com.tzld.longarticle.recommend.server.service.score.strategy.ContentPoolStrategy;
import com.tzld.longarticle.recommend.server.service.score.strategy.SimilarityStrategy;
import com.tzld.longarticle.recommend.server.service.score.strategy.ViewCountStrategy;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

/**
 * @author dyp
 */
@Service
@Slf4j
public class RankService {

    @Autowired
    private ScoreService scoreService;

    public RankResult rank(RankParam param) {

        ScoreResult result = scoreService.score(convertToScoreParam(param));
        Map<String, Map<Class<? extends ScoreStrategy>, Double>> scoreMap = result.getScoreMap();

        List<RankItem> items = CommonCollectionUtils.toList(param.getContents(), c -> {
            Map<Class<? extends ScoreStrategy>, Double> map = scoreMap.get(c.getId());
            double score = 1 * map.getOrDefault(SimilarityStrategy.class, 0.0)
                    + 10 * map.getOrDefault(ViewCountStrategy.class, 0.0)
                    + 100 * map.getOrDefault(ContentPoolStrategy.class, 0.0);

            RankItem item = new RankItem();
            item.setContent(c);
            item.setScore(score);
            return item;

        });
        Collections.sort(items, Comparator.comparingDouble(o -> -o.getScore()));
        return new RankResult(items);
    }

    private ScoreParam convertToScoreParam(RankParam param) {
        ScoreParam scoreParam = new ScoreParam();
        scoreParam.setAccountName(param.getAccountName());
        scoreParam.setContents(param.getContents());
        return scoreParam;
    }

}
